package com.qmth.themis.business.service.impl;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import javax.annotation.Resource;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.themis.business.bean.exam.ExamPrepareBean;
import com.qmth.themis.business.cache.RedisKeyHelper;
import com.qmth.themis.business.cache.bean.ExamActivityCacheBean;
import com.qmth.themis.business.cache.bean.ExamStudentCacheBean;
import com.qmth.themis.business.dao.TEExamMapper;
import com.qmth.themis.business.entity.TEExam;
import com.qmth.themis.business.service.TEExamActivityService;
import com.qmth.themis.business.service.TEExamService;
import com.qmth.themis.business.service.TEExamStudentService;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.common.exception.BusinessException;

/**
 * @Description: 考试批次 服务实现类
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Service
public class TEExamServiceImpl extends ServiceImpl<TEExamMapper, TEExam> implements TEExamService {

    @Resource
    TEExamMapper teExamMapper;

    @Resource
    TEExamActivityService teExamActivityService;
    
    @Resource
    TEExamStudentService teExamStudentService;

    @Resource
    RedisUtil redisUtil;
    /**
     * 查询考试批次
     *
     * @param iPage
     * @param id
     * @param code
     * @param name
     * @param mode
     * @param enable
     * @return
     */
    @Override
    public IPage<TEExam> examQuery(IPage<Map> iPage, Long id, String code, String name, Integer mode, Integer enable) {
        return teExamMapper.examQuery(iPage, id, code, name, mode, enable);
    }

    /**
     * 获取考试待考列表
     *
     * @param studentId
     * @return
     */
    @Override
    public List<Map> getWaitingExam(Long studentId) {
        List<Map> list = teExamMapper.getWaitingExam(studentId);
        if (Objects.nonNull(list) && list.size() > 0) {
            for (int i = 0; i < list.size(); i++) {
                Map m = list.get(i);
                List<Map> teExamActivityList = teExamActivityService.getWaitingExam(studentId, Long.parseLong(String.valueOf(m.get("id"))), Long.parseLong(String.valueOf(m.get("examActivityId"))));
                m.put("activities", teExamActivityList);
            }
        }
        return list;
    }

    @Transactional
	@Override
	public ExamPrepareBean prepare(Long studentId, Long examStudentId) {
    	ExamPrepareBean ret=null;
    	ExamStudentCacheBean es=null;
    	es=(ExamStudentCacheBean)redisUtil.get(RedisKeyHelper.examStudentCacheKey(examStudentId));
    	if(es==null) {
    		es=teExamStudentService.getExamStudnetCacheBean(examStudentId);
    	}
    	if(es==null) {
    		throw new BusinessException("未找到考试");
    	}
    	if(studentId.equals(es.getStudentId())) {
    		throw new BusinessException("考生Id和当前登录用户不一致");
    	}
    	if(es.getLeftExamCount()==0) {
    		throw new BusinessException("没有剩余考试次数");
    	}
    	Long activityId=es.getExamActivityId();
    	ExamActivityCacheBean ac=teExamActivityService.getExamActivityCacheBean(activityId);
    	if(ac==null) {
    		throw new BusinessException("未找到场次");
    	}
    	Date now=new Date();
    	Long start=ac.getStartTime().getTime()-(ac.getPrepareSeconds()*1000);
    	Long end=ac.getStartTime().getTime()+(ac.getOpeningSeconds()*1000);
    	if(now.getTime()<start) {
    		throw new BusinessException("没有到允许开考的时间");
    	}
    	if(now.getTime()>end) {
    		throw new BusinessException("允许开考的时间已结束");
    	}
		return ret;
	}
}
